"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var CleanWebpackPlugin = require("clean-webpack-plugin");
var CopyWebpackPlugin = require("copy-webpack-plugin");
var FriendlyErrorsWebpackPlugin = require("friendly-errors-webpack-plugin");
var HtmlWebpackPlugin = require("html-webpack-plugin");
var MiniCssExtractPlugin = require("mini-css-extract-plugin");
var OfflinePlugin = require("offline-plugin");
var OpenBrowserPlugin = require("open-browser-webpack-plugin");
var OptimizeCssAssetsPlugin = require("optimize-css-assets-webpack-plugin");
var path_1 = require("path");
var SpeedMeasurePlugin = require("speed-measure-webpack-plugin");
var UglifyJsPlugin = require("uglifyjs-webpack-plugin");
var webpack = require("webpack");
var webpack_bundle_analyzer_1 = require("webpack-bundle-analyzer");
var webpackMerge = require("webpack-merge");
var nodeExternals = require("webpack-node-externals");
var WebpackBar = require("webpackbar");
var rootPath = path_1.resolve(process.cwd());
var context = rootPath;
var loader = function (name) { return require.resolve(name); };
var makeDevEntry = function (entry, port) {
    return [
        require.resolve("webpack-dev-server/client") + ("?http://localhost:" + port),
        require.resolve("webpack/hot/dev-server"),
        entry
    ];
};
var resolveLocal = function () {
    var args = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        args[_i] = arguments[_i];
    }
    return path_1.join.apply(void 0, [__dirname, "../"].concat(args));
};
var resolveContext = function (path) {
    return path_1.resolve(context, path);
};
var isBackEndConfig = function (config) {
    return config.mode && (config.mode === "server" || config.mode === "cli");
};
var isCliConfig = function (config) { return config.mode && config.mode === "cli"; };
var configuration = function (userConfig, env, isDebugging) {
    if (env === void 0) { env = "development"; }
    if (isDebugging === void 0) { isDebugging = false; }
    var isDev = env !== "production";
    var port = userConfig.port || 8080;
    var paths = __assign({ src: path_1.join(rootPath, "./src"), test: path_1.join(rootPath, "./test"), entry: {
            main: path_1.join(rootPath, "./src/index")
        }, htmlTemplate: path_1.join(rootPath, "./src/index.html"), outputFolder: path_1.join(rootPath, "./dist") }, userConfig.paths);
    var getPresets = function () { return ({
        rules: {
            typescript: {
                test: /\.tsx?$/,
                use: [
                    {
                        loader: loader("ts-loader"),
                        options: { context: rootPath }
                    }
                ],
                include: paths.src,
                exclude: /node_modules/
            },
            babel: {
                test: /\.jsx?$/,
                loader: loader("babel-loader"),
                include: paths.src,
                exclude: /node_modules/
            },
            html: {
                test: /\.(html)$/,
                loader: loader("html-loader"),
                include: paths.src,
                exclude: /node_modules/
            },
            scss: {
                test: /\.(sa|sc|c)ss$/,
                use: [
                    isDev ? loader("style-loader") : MiniCssExtractPlugin.loader,
                    {
                        loader: loader("css-loader"),
                        options: { sourceMap: !isDev, importLoaders: 1 }
                    },
                    {
                        loader: loader("postcss-loader"),
                        options: { sourceMap: !isDev }
                    },
                    {
                        loader: loader("sass-loader"),
                        options: { sourceMap: !isDev }
                    }
                ]
            },
            images: {
                test: /\.(png|svg|jpg|gif)$/,
                loader: loader("url-loader"),
                options: { limit: 10000, name: "images/[name].[hash:7].[ext]" }
            },
            fonts: {
                test: /\.(woff|woff2|eot|ttf|otf)$/,
                loader: loader("url-loader"),
                options: { limit: 10000, name: "fonts/[name].[hash:7].[ext]" }
            },
            media: {
                test: /\.(mp4|webm|ogg|mp3|wav|flac|aac)(\?.*)?$/,
                loader: loader("url-loader"),
                options: { limit: 10000, name: "media/[name].[hash:7].[ext]" }
            }
        },
        plugins: {
            openBrowser: new OpenBrowserPlugin({ url: "http://localhost:" + port }),
            webpackbar: new WebpackBar(),
            offline: new OfflinePlugin(),
            friendlyErrors: new FriendlyErrorsWebpackPlugin(),
            miniCssExtract: new MiniCssExtractPlugin({
                filename: "css/[name].[contenthash].css",
                chunkFilename: "css/[id].[chunkhash].css"
            }),
            html: new HtmlWebpackPlugin({
                filename: "index.html",
                template: paths.htmlTemplate,
                inject: true,
                minify: isDev
                    ? false
                    : {
                        removeComments: true,
                        collapseWhitespace: true,
                        removeAttributeQuotes: true
                    }
            }),
            clean: new CleanWebpackPlugin(["dist"], {
                root: rootPath
            }),
            hmr: new webpack.HotModuleReplacementPlugin(),
            analyze: new webpack_bundle_analyzer_1.BundleAnalyzerPlugin(),
            copyStatic: new CopyWebpackPlugin([
                {
                    from: path_1.join(rootPath, "static"),
                    to: paths.outputFolder,
                    ignore: [".*"]
                }
            ]),
            banner: new webpack.BannerPlugin({
                banner: "#!/usr/bin/env node",
                raw: true
            })
        }
    }); };
    var getBaseConfig = function () {
        var rules = mergedConfig.rules, plugins = mergedConfig.plugins;
        return {
            context: context,
            mode: isDev ? "development" : "production",
            devtool: isDev ? "inline-source-map" : "source-map",
            resolve: {
                extensions: [".tsx", ".ts", ".js", ".jsx", ".json"],
                modules: [
                    "node_modules",
                    resolveContext("node_modules"),
                    resolveLocal("node_modules")
                ]
            },
            resolveLoader: {
                modules: [
                    "node_modules",
                    resolveContext("node_modules"),
                    resolveLocal("node_modules")
                ]
            },
            module: {
                rules: [rules.typescript, rules.babel]
            },
            plugins: [
                plugins.friendlyErrors,
                plugins.webpackbar,
                plugins.clean
            ].concat((isDev ? [plugins.hmr] : []), (process.env.BUNDLE_ANALYZE ? [plugins.analyze] : []), (process.env.OFFLINE_PLUGIN ? [plugins.offline] : [])),
            optimization: isDev
                ? {}
                : {
                    minimizer: [
                        new UglifyJsPlugin({
                            cache: true,
                            parallel: true,
                            sourceMap: true
                        })
                    ]
                }
        };
    };
    var getBackEndConfig = function () {
        return webpackMerge(getBaseConfig(), {
            target: "node",
            externals: [nodeExternals()],
            entry: mergedConfig.paths.entry,
            output: {
                filename: "index.js",
                path: path_1.join(rootPath, "./dist"),
                libraryTarget: "commonjs2"
            }
        });
    };
    var getWebAppConfig = function () {
        var rules = mergedConfig.rules, plugins = mergedConfig.plugins;
        return webpackMerge(getBaseConfig(), {
            entry: isDev
                ? { main: makeDevEntry(paths.entry.main, port) }
                : paths.entry,
            module: {
                rules: [rules.html, rules.scss, rules.images, rules.fonts, rules.media]
            },
            plugins: [
                plugins.html,
                plugins.copyStatic
            ].concat((isDev ? [plugins.openBrowser] : [plugins.miniCssExtract])),
            output: {
                publicPath: "/",
                filename: isDev ? "js/[name].js" : "js/[name].[chunkhash].js",
                chunkFilename: isDev ? "js/[id].js" : "js/[id].[chunkhash].js",
                path: paths.outputFolder,
                libraryTarget: "umd"
            },
            optimization: isDev
                ? {}
                : {
                    minimizer: [
                        new OptimizeCssAssetsPlugin({
                            cssProcessor: require("cssnano"),
                            cssProcessorPluginOptions: {
                                preset: [
                                    "default",
                                    {
                                        discardComments: {
                                            removeAll: true
                                        }
                                    }
                                ]
                            },
                            canPrint: true
                        })
                    ]
                }
        });
    };
    var defaultConfig = getPresets();
    var mergedConfig = __assign({}, defaultConfig, userConfig.webpack);
    var resultConfig = getWebAppConfig();
    if (isBackEndConfig(mergedConfig)) {
        resultConfig = getBackEndConfig();
    }
    if (isCliConfig(mergedConfig)) {
        resultConfig.plugins.push(mergedConfig.plugins.banner);
    }
    if (isDebugging) {
        console.log(JSON.stringify(resultConfig));
    }
    var smp = new SpeedMeasurePlugin({
        disable: !process.env.SMP
    });
    return smp.wrap(resultConfig);
};
exports.default = configuration;
//# sourceMappingURL=config.js.map